@@ -417,9 +417,9 @@ about_dialog_timer (gpointer state_)
 }
 
 static gboolean
-about_dialog_anim_draw (GtkWidget *widget,
-                          cairo_t *cr,
-                          AboutState *state)
+about_dialog_anim_draw (G_GNUC_UNUSED GtkWidget *widget,
+			cairo_t *cr,
+			AboutState *state)
 {
 	GList *l;
 
@@ -569,7 +569,7 @@ dialog_about (WBCGtk *wbcg)
 			  "website", "http://www.gnumeric.org/",
 			  "website-label", _("Visit the Gnumeric website"),
 			  "logo-icon-name", "gnumeric",
-			  "copyright", _("Copyright \xc2\xa9 1998-2010"),
+			  "copyright", _("Copyright \xc2\xa9 1998-2011"),
 			  "comments", _("Free, Fast, Accurate - Pick Any Three!"),
 			  NULL);
 	state->dialog = w;
