@@ -25,6 +25,7 @@
 #include "sheet-style.h"
 #include "sheet-object.h"
 #include "sheet-control-gui.h"
+#include "sheet-view.h"
 #include "commands.h"
 #include "value.h"
 #include "number-match.h"
@@ -32,7 +33,7 @@
 #include "stf-parse.h"
 #include "mstyle.h"
 #include "gnm-format.h"
-#include "gnumeric-gconf.h"
+#include "gnumeric-conf.h"
 #include "xml-sax.h"
 #include "gutils.h"
 
@@ -617,19 +618,15 @@ x_targets_received (GtkClipboard *clipboard, GdkAtom *targets,
 
 	if (table_atom == GDK_NONE) {
 		GtkTargetList *tl = gtk_target_list_new (NULL, 0);
-		GList *l;
 		gboolean found = FALSE;
 
 		gtk_target_list_add_image_targets (tl, 0, FALSE);
 
 		/* Find an image format */
 		for (i = 0 ; i < n_targets && !found; i++) {
-			for (l = tl->list; l && !found; l = l->next) {
-				GtkTargetPair *tp = l->data;
-				if (tp->target == targets[i]) {
-					ctxt->image_atom = targets[i];
-					found = TRUE;
-				}
+			if (gtk_target_list_find (tl, targets[i], NULL)) {
+				ctxt->image_atom = targets[i];
+				found = TRUE;
 			}
 		}
 		gtk_target_list_unref (tl);
@@ -786,7 +783,7 @@ image_write (GnmCellRegion *cr, gchar const *mime_type, int *size)
 }
 
 static guchar *
-graph_write (GnmCellRegion *cr, gchar const *mime_type, int *size)
+object_write (GnmCellRegion *cr, gchar const *mime_type, int *size)
 {
 	guchar *ret = NULL;
 	SheetObject *so = NULL;
@@ -896,9 +893,10 @@ x_clipboard_get_cb (GtkClipboard *gclipboard, GtkSelectionData *selection_data,
 					target, 8,
 					(guchar *) buffer, buffer_size);
 		g_free (buffer);
-	} else if (strcmp (target_name, "application/x-goffice-graph") == 0) {
+	} else if (strcmp (target_name, "application/x-goffice-graph") == 0 ||
+	           g_slist_find_custom (go_components_get_mime_types (), target_name, (GCompareFunc) strcmp) != NULL) {
 		int buffer_size;
-		guchar *buffer = graph_write (clipboard, target_name,
+		guchar *buffer = object_write (clipboard, target_name,
 					      &buffer_size);
 		if (debug_clipboard ())
 			g_message ("clipboard graph of %d bytes",
@@ -949,14 +947,23 @@ x_clipboard_get_cb (GtkClipboard *gclipboard, GtkSelectionData *selection_data,
 
 		/* If the other app was a gnumeric, emulate a cut */
 		if (to_gnumeric) {
-			/* FIXME: This needs undo.
-			 * Use a variant of cmd_selection_clear that would use
-			 * the src range rather than the current selection */
-			sheet_clear_region (sheet,
-				a->start.col, a->start.row,
-				a->end.col,   a->end.row,
-				CLEAR_VALUES|CLEAR_COMMENTS|CLEAR_RECALC_DEPS,
-				ctx);
+			GOUndo *redo, *undo;
+			GnmSheetRange *sr    = gnm_sheet_range_new (sheet, a);
+			SheetView const *sv  = gnm_app_clipboard_sheet_view_get ();
+			SheetControl *sc     = g_ptr_array_index (sv->controls, 0);
+			WorkbookControl *wbc = sc_wbc (sc);
+			char *name;
+			char *text;
+
+			redo = sheet_clear_region_undo
+				(sr,
+				 CLEAR_VALUES|CLEAR_COMMENTS|CLEAR_RECALC_DEPS);
+			undo = clipboard_copy_range_undo (sheet, a);
+			name = undo_range_name (sheet, a);
+			text = g_strdup_printf (_("Cut of %s"), name);
+			g_free (name);
+			cmd_generic (wbc, text, undo, redo);
+			g_free (text);
 			gnm_app_clipboard_clear (TRUE);
 		}
 
@@ -1005,32 +1012,6 @@ gnm_x_request_clipboard (WBCGtk *wbcg, GnmPasteTarget const *pt)
 				       x_targets_received, ctxt);
 }
 
-static GtkTargetEntry*
-target_list_to_entries (GtkTargetList *target_list, int *n_entries)
-{
-	GtkTargetEntry *entries;
-	int n_targets, i;
-	GList *p;
-
-	if (!target_list || !target_list->list || n_entries == NULL)
-		return NULL;
-
-	n_targets = g_list_length (target_list->list);
-	if (n_targets == 0)
-		return NULL;
-
-	entries = g_new0 (GtkTargetEntry, n_targets);
-	for (p = target_list->list, i = 0; p != NULL; p = p->next, i++) {
-		GtkTargetPair *tp = p->data;
-		entries[i].target = gdk_atom_name (tp->target);
-		entries[i].flags  = tp->flags;
-		entries[i].info   = tp->info;
-	}
-
-	*n_entries = n_targets;
-	return entries;
-}
-
 /* Restrict the	set of formats offered to clipboard manager. */
 /* We include bmp in the whitelist because that's the only image format
  * we share with OOo over clipboard (!) */
@@ -1084,10 +1065,7 @@ gnm_x_claim_clipboard (WBCGtk *wbcg)
 	GnmCellRegion *content = gnm_app_clipboard_contents_get ();
 	SheetObject *imageable = NULL, *exportable = NULL;
 	GtkTargetEntry *targets = NULL;
-	int n_targets, i;
-	GSList *ptr;
-	SheetObject *candidate;
-	GtkTargetList *tl;
+	int n_targets;
 	gboolean ret;
 
 	static GtkTargetEntry const table_targets[] = {
@@ -1102,9 +1080,10 @@ gnm_x_claim_clipboard (WBCGtk *wbcg)
 	n_targets = G_N_ELEMENTS (table_targets);
 	if (content &&
 	    (content->cols <= 0 || content->rows <= 0)) {
+		GSList *ptr;
 		for (ptr = content->objects; ptr != NULL;
 		     ptr = ptr->next) {
-			candidate = SHEET_OBJECT (ptr->data);
+			SheetObject *candidate = SHEET_OBJECT (ptr->data);
 			if (exportable == NULL && IS_SHEET_OBJECT_EXPORTABLE (candidate))
 				exportable = candidate;
 			if (imageable == NULL && IS_SHEET_OBJECT_IMAGEABLE (candidate))
@@ -1114,17 +1093,19 @@ gnm_x_claim_clipboard (WBCGtk *wbcg)
 		n_targets = 1;
 	}
 	if (exportable) {
-		tl = sheet_object_exportable_get_target_list (exportable);
+		GtkTargetList *tl =
+			sheet_object_exportable_get_target_list (exportable);
 		/* _add_table prepends to target_list */
 		gtk_target_list_add_table (tl, table_targets, 1);
-		targets = target_list_to_entries (tl, &n_targets);
+		targets = gtk_target_table_new_from_list (tl, &n_targets);
 		gtk_target_list_unref (tl);
 	}
 	if (imageable) {
-		tl = sheet_object_get_target_list (imageable);
+		GtkTargetList *tl =
+			sheet_object_get_target_list (imageable);
 		/* _add_table prepends to target_list */
 		gtk_target_list_add_table (tl, targets, (exportable)? n_targets: 1);
-		targets = target_list_to_entries (tl, &n_targets);
+		targets = gtk_target_table_new_from_list (tl, &n_targets);
 		gtk_target_list_unref (tl);
 	}
 	/* Register a x_clipboard_clear_cb only for CLIPBOARD, not for
@@ -1145,11 +1126,8 @@ gnm_x_claim_clipboard (WBCGtk *wbcg)
 			NULL,
 			gnm_app_get_app ());
 	}
-	if (exportable || imageable) {
-		for (i = 0; i < n_targets; i++)
-			g_free (targets[i].target);
-		g_free (targets);
-	}
+	if (exportable || imageable)
+		gtk_target_table_free (targets, n_targets);
 
 	return ret;
 }
