@@ -175,6 +175,148 @@ gnm_usr_dir (gboolean versioned)
 	return versioned ? gnumeric_usr_dir : gnumeric_usr_dir_unversioned;
 }
 
+/*
+ * Like strto[ld], but...
+ * 1. handles non-ascii characters
+ * 2. disallows 0x000.0p+00 and 0.0d+00
+ * 3. ensures sane errno on exit
+ */
+gnm_float
+gnm_utf8_strto (const char *s, char **end)
+{
+	const char *p;
+	int sign;
+	char *dummy_end;
+	GString *ascii = g_string_sized_new (100);
+	GString const *decimal = go_locale_get_decimal ();
+	gboolean seen_decimal = FALSE;
+	gboolean seen_digit = FALSE;
+	size_t spaces = 0;
+	gnm_float res;
+	int save_errno;
+
+	if (!end)
+		end = &dummy_end;
+
+	p = s;
+	while (g_unichar_isspace (g_utf8_get_char (p))) {
+		p = g_utf8_next_char (p);
+		spaces++;
+	}
+
+	sign = go_unichar_issign (g_utf8_get_char (p));
+	if (sign) {
+		g_string_append_c (ascii, "-/+"[sign + 1]);
+		p = g_utf8_next_char (p);
+	}
+
+	do {
+		if (strncmp (p, decimal->str, decimal->len) == 0) {
+			if (seen_decimal)
+				break;
+			seen_decimal = TRUE;
+			go_string_append_gstring (ascii, decimal);
+			p += decimal->len;
+		} else if (g_unichar_isdigit (g_utf8_get_char (p))) {
+			g_string_append_c (ascii, '0' + g_unichar_digit_value (g_utf8_get_char (p)));
+			p = g_utf8_next_char (p);
+			seen_digit = TRUE;
+		} else
+			break;
+	} while (1);
+
+	if (!seen_digit) {
+		/* No conversion, bail to gnm_strto for nan etc. */
+		g_string_free (ascii, TRUE);
+		return gnm_strto (s, end);
+	}
+
+	if (*p == 'e' || *p == 'E') {
+		int sign;
+
+		g_string_append_c (ascii, 'e');
+		p = g_utf8_next_char (p);
+
+		sign = go_unichar_issign (g_utf8_get_char (p));
+		if (sign) {
+			g_string_append_c (ascii, "-/+"[sign + 1]);
+			p = g_utf8_next_char (p);
+		}
+		while (g_unichar_isdigit (g_utf8_get_char (p))) {
+			g_string_append_c (ascii, '0' + g_unichar_digit_value (g_utf8_get_char (p)));
+			p = g_utf8_next_char (p);
+		}
+	}
+
+	res = gnm_strto (ascii->str, end);
+	save_errno = errno;
+	*end = g_utf8_offset_to_pointer
+		(s, spaces + g_utf8_pointer_to_offset (ascii->str, *end));
+	g_string_free (ascii, TRUE);
+
+	errno = save_errno;
+	return res;
+}
+
+/*
+ * Like strtol, but...
+ * 1. handles non-ascii characters
+ * 2. assumes base==10
+ * 3. ensures sane errno on exit
+ */
+long
+gnm_utf8_strtol (const char *s, char **end)
+{
+	const char *p;
+	int sign;
+	char *dummy_end;
+	unsigned long res = 0, lim, limd;
+
+	if (!end)
+		end = &dummy_end;
+
+	p = s;
+	while (g_unichar_isspace (g_utf8_get_char (p)))
+		p = g_utf8_next_char (p);
+
+	sign = go_unichar_issign (g_utf8_get_char (p));
+	if (sign)
+		p = g_utf8_next_char (p);
+	if (sign < 0) {
+		lim = (-(unsigned long)LONG_MIN) / 10u;
+		limd = (-(unsigned long)LONG_MIN) % 10u;
+	} else {
+		lim = (unsigned long)LONG_MAX / 10u;
+		limd = (unsigned long)LONG_MAX % 10u;
+	}
+
+	if (!g_unichar_isdigit (g_utf8_get_char (p))) {
+		errno = 0;
+		*end = (char *)s;
+		return 0;
+	}
+
+	while (g_unichar_isdigit (g_utf8_get_char (p))) {
+		guint8 dig = g_unichar_digit_value (g_utf8_get_char (p));
+		p = g_utf8_next_char (p);
+
+		if (res > lim || (res == lim && dig > limd)) {
+			/* Overflow */
+			while (g_unichar_isdigit (g_utf8_get_char (p)))
+				p = g_utf8_next_char (p);
+			*end = (char *)p;
+			errno = ERANGE;
+			return sign < 0 ? LONG_MIN : LONG_MAX;
+		}
+
+		res = res * 10u + dig;
+	}
+	*end = (char *)p;
+	errno = 0;
+	return sign < 0 ? (long)-res : (long)res;
+}
+
+
 int
 gnm_regcomp_XL (GORegexp *preg, char const *pattern, int cflags,
 		gboolean full)
