@@ -3,6 +3,7 @@
  * workbook-view.c: View functions for the workbook
  *
  * Copyright (C) 2000-2006 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2012 Morten Welinder (terra@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -30,8 +31,6 @@
 #include "sheet-view.h"
 #include "sheet-merge.h"
 #include "sheet-style.h"
-#include "style-font.h"
-#include "gnm-format.h"
 #include "func.h"
 #include "expr.h"
 #include "expr-name.h"
@@ -75,10 +74,8 @@ enum {
 	PROP_AUTO_EXPR_FUNC,
 	PROP_AUTO_EXPR_DESCR,
 	PROP_AUTO_EXPR_MAX_PRECISION,
-	PROP_AUTO_EXPR_TEXT,
-	PROP_AUTO_EXPR_ATTRS,
-	PROP_AUTO_EXPR_CELL,
-	PROP_AUTO_EXPR_SHEET,
+	PROP_AUTO_EXPR_VALUE,
+	PROP_AUTO_EXPR_EVAL_POS,
 	PROP_SHOW_HORIZONTAL_SCROLLBAR,
 	PROP_SHOW_VERTICAL_SCROLLBAR,
 	PROP_SHOW_NOTEBOOK_TABS,
@@ -369,7 +366,7 @@ wb_view_selection_desc (WorkbookView *wbv, gboolean use_pos,
 		g_return_if_fail (IS_SHEET_VIEW (sv));
 		g_return_if_fail (sv->selections);
 
-		r = sv->selections->data;
+		r = selection_first_range (sv, NULL, NULL);
 
 		if (use_pos || range_is_singleton (r) ||
 		    (NULL != (m = gnm_sheet_merge_is_corner (sv->sheet, &r->start)) &&
@@ -428,7 +425,7 @@ wb_view_edit_line_set (WorkbookView *wbv, WorkbookControl *optional_wbc)
 			sv->edit_pos.col, sv->edit_pos.row);
 
 		if (NULL != cell) {
-			text = gnm_cell_get_entered_text (cell);
+			text = gnm_cell_get_text_for_editing (cell, sheet, NULL, NULL);
 
 			if (gnm_cell_has_expr (cell)) {
 				GnmExprTop const *texpr = cell->base.texpr;
@@ -498,13 +495,11 @@ accumulate_regions (SheetView *sv,  GnmRange const *r, gpointer closure)
 void
 wb_view_auto_expr_recalc (WorkbookView *wbv)
 {
-	GnmEvalPos      ep;
 	GnmExprList	*selection = NULL;
 	GnmValue	*v;
 	SheetView	*sv;
 	GnmExprTop const *texpr;
-	GnmRange        r;
-	GString *str;
+	GnmEvalPos      ep;
 
 	g_return_if_fail (IS_WORKBOOK_VIEW (wbv));
 
@@ -513,111 +508,39 @@ wb_view_auto_expr_recalc (WorkbookView *wbv)
 	    sv == NULL)
 		return;
 
-	if (wbv->auto_expr_sheet != NULL &&
-	    wbv->auto_expr_descr != NULL &&
-	    wbv->auto_expr_cell.row >= 0 &&
-	    wbv->auto_expr_cell.col >= 0) {
-		/* We need to check that wbv->auto_expr_sheet is still valid */
-		GSList *sheets = workbook_sheets (wbv->wb);
-		if (g_slist_find (sheets, wbv->auto_expr_sheet) == NULL) {
-			v = value_new_error_REF (NULL);
-			str = g_string_new ("?!");
-		} else {
-			range_init_cellpos (&r, &wbv->auto_expr_cell);
-			v = value_new_cellrange_r (wbv->auto_expr_sheet, &r);
-			if (strlen (wbv->auto_expr_sheet->name_unquoted) < 8) {
-				str = g_string_new
-					(wbv->auto_expr_sheet->name_unquoted);
-				g_string_append_c (str, '!');
-			} else
-				str = g_string_new ("\342\200\246!");
-		}
-		texpr = gnm_expr_top_new_constant (v);
-		g_string_append (str, wbv->auto_expr_descr);
-	} else if (wbv->auto_expr_func != NULL &&
-		   wbv->auto_expr_descr != NULL) {
+	if (wbv->auto_expr.dep.sheet != NULL &&
+	    wbv->auto_expr.dep.texpr != NULL) {
+		texpr = wbv->auto_expr.dep.texpr;
+		gnm_expr_top_ref (texpr);
+	} else if (wbv->auto_expr.func != NULL) {
 		sv_selection_apply (sv, &accumulate_regions, FALSE, &selection);
 		texpr = gnm_expr_top_new
-			(gnm_expr_new_funcall (wbv->auto_expr_func, selection));
-		str = g_string_new (wbv->auto_expr_descr);
-	} else
-		return;
+			(gnm_expr_new_funcall (wbv->auto_expr.func, selection));
+	} else {
+		texpr = gnm_expr_top_new_constant (value_new_string (""));
+	}
 
 	eval_pos_init_sheet (&ep, wbv->current_sheet);
+
 	v = gnm_expr_top_eval (texpr, &ep, GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
-
 	if (v) {
-		GOFormat const *format = NULL;
-		GOFormat const *tmp_format = NULL;
-		PangoAttrList *attrs = NULL;
-
-		g_string_append_c (str, '=');
-		if (!wbv->auto_expr_use_max_precision) {
-			format = VALUE_FMT (v);
-			if (!format)
-				format = tmp_format =
-					auto_style_format_suggest (texpr, &ep);
+		if (wbv->auto_expr.use_max_precision)
+			value_set_fmt (v, NULL);
+		else if (!VALUE_FMT (v)) {
+			GOFormat const *fmt = auto_style_format_suggest (texpr, &ep);
+			value_set_fmt (v, fmt);
+			go_format_unref (fmt);
 		}
-
-		if (format) {
-			PangoContext *context = gnm_pango_context_get ();
-			PangoLayout *layout = pango_layout_new (context);
-			gsize old_len = str->len;
-			GOFormatNumberError err =
-				format_value_layout (layout, format, v,
-						     /* Note that we created a label large enough for */
-						     /* "Sumerage=-012345678901234" */
-						     25 - g_utf8_strlen (str->str, -1),
-						     workbook_date_conv (wb_view_get_workbook (wbv)));
-			go_format_unref (tmp_format);
-			switch (err) {
-			case GO_FORMAT_NUMBER_OK:
-			case GO_FORMAT_NUMBER_DATE_ERROR: {
-				PangoAttrList *atl;
-
-				go_pango_translate_layout (layout); /* translating custom attributes */
-				g_string_append (str, pango_layout_get_text (layout));
-				/* We need to shift the attribute list  */
-				atl = pango_attr_list_ref (pango_layout_get_attributes (layout));
-				if (atl != NULL) {
-					attrs = pango_attr_list_new ();
-					pango_attr_list_splice
-						(attrs, atl, old_len,
-						 str->len - old_len);
-					pango_attr_list_unref (atl);
-				}
-				break;
-			}
-			default:
-			case GO_FORMAT_NUMBER_INVALID_FORMAT:
-				g_string_append (str,  _("Invalid format"));
-				break;
-			}
-			g_object_unref (layout);
-			g_object_unref (context);
-		} else
-			g_string_append (str, value_peek_string (v));
-
-		g_object_set (wbv,
-			      "auto-expr-text", str->str,
-			      "auto-expr-attrs", attrs,
-			      NULL);
-		pango_attr_list_unref (attrs);
-		value_release (v);
-	} else {
-		g_object_set (wbv,
-			      "auto-expr-text", "Internal ERROR",
-			      "auto-expr-attrs", NULL,
-			      NULL);
 	}
-	g_string_free (str, TRUE);
+	g_object_set (wbv, "auto-expr-value", v, NULL);
+	value_release (v);
 	gnm_expr_top_unref (texpr);
 }
 
 /* perform whatever initialization of a control that is necessary when it
  * finally gets assigned to a view with a workbook */
 static void
-wb_view_init_control (WorkbookControl *wbc)
+wb_view_init_control (G_GNUC_UNUSED WorkbookControl *wbc)
 {
 }
 
@@ -656,15 +579,15 @@ static GObjectClass *parent_class;
 static void
 wb_view_auto_expr_func (WorkbookView *wbv, GnmFunc *func)
 {
-	if (wbv->auto_expr_func == func)
+	if (wbv->auto_expr.func == func)
 		return;
 
-	if (wbv->auto_expr_func)
-		gnm_func_unref (wbv->auto_expr_func);
+	if (wbv->auto_expr.func)
+		gnm_func_unref (wbv->auto_expr.func);
 
 	if (func)
 		gnm_func_ref (func);
-	wbv->auto_expr_func = func;
+	wbv->auto_expr.func = func;
 
 	wb_view_auto_expr_recalc (wbv);
 }
@@ -674,12 +597,12 @@ wb_view_auto_expr_descr (WorkbookView *wbv, const char *descr)
 {
 	char *s;
 
-	if (go_str_compare (descr, wbv->auto_expr_descr) == 0)
+	if (go_str_compare (descr, wbv->auto_expr.descr) == 0)
 		return;
 
 	s = g_strdup (descr);
-	g_free (wbv->auto_expr_descr);
-	wbv->auto_expr_descr = s;
+	g_free (wbv->auto_expr.descr);
+	wbv->auto_expr.descr = s;
 
 	wb_view_auto_expr_recalc (wbv);
 }
@@ -689,62 +612,63 @@ wb_view_auto_expr_precision (WorkbookView *wbv, gboolean use_max_precision)
 {
 	use_max_precision = !!use_max_precision;
 
-	if (wbv->auto_expr_use_max_precision == use_max_precision)
+	if (wbv->auto_expr.use_max_precision == use_max_precision)
 		return;
 
-	wbv->auto_expr_use_max_precision = use_max_precision;
+	wbv->auto_expr.use_max_precision = use_max_precision;
 
 	wb_view_auto_expr_recalc (wbv);
 }
 
 static void
-wb_view_auto_expr_text (WorkbookView *wbv, const char *text)
+wb_view_auto_expr_value (WorkbookView *wbv, const GnmValue *value)
 {
-	char *s;
-
-	if (go_str_compare (text, wbv->auto_expr_text) == 0)
-		return;
-
-	s = g_strdup (text);
-	g_free (wbv->auto_expr_text);
-	wbv->auto_expr_text = s;
+	value_release (wbv->auto_expr.value);
+	wbv->auto_expr.value = value_dup (value);
 }
 
 static void
-wb_view_auto_expr_attrs (WorkbookView *wbv, PangoAttrList *attrs)
+cb_clear_auto_expr_sheet (WorkbookView *wbv)
 {
-	if (gnm_pango_attr_list_equal (attrs, wbv->auto_expr_attrs))
-		return;
-
-	if (attrs)
-		pango_attr_list_ref (attrs);
-	if (wbv->auto_expr_attrs)
-		pango_attr_list_unref (wbv->auto_expr_attrs);
-	wbv->auto_expr_attrs = attrs;
+	g_object_set (G_OBJECT (wbv),
+		      "auto-expr-eval-pos", NULL,
+		      NULL);
 }
 
 static void
-wb_view_auto_expr_cell (WorkbookView *wbv, gpointer *cell)
+wb_view_auto_expr_eval_pos (WorkbookView *wbv, GnmEvalPos const *ep)
 {
-	if (cell == NULL) {
-		wbv->auto_expr_cell.col = -1;
-		wbv->auto_expr_cell.row = -1;
-	} else {
-		wbv->auto_expr_cell = *((GnmCellPos *)cell);
-		wb_view_auto_expr_recalc (wbv);
+	Sheet *sheet = ep ? ep->sheet : NULL;
+
+	if (wbv->auto_expr.sheet_detached_sig) {
+		g_signal_handler_disconnect (wbv->auto_expr.dep.sheet,
+					     wbv->auto_expr.sheet_detached_sig);
+		wbv->auto_expr.sheet_detached_sig = 0;
 	}
+
+	dependent_set_expr (&wbv->auto_expr.dep, NULL);
+	dependent_managed_set_sheet (&wbv->auto_expr.dep, sheet);
+
+	if (sheet) {
+		GnmRange r;
+		GnmValue *v;
+		GnmExprTop const *texpr;
+
+		wbv->auto_expr.sheet_detached_sig = g_signal_connect_swapped (
+			G_OBJECT (sheet),
+			"detached-from-workbook",
+			G_CALLBACK (cb_clear_auto_expr_sheet), wbv);
+
+		range_init_cellpos (&r, &ep->eval);
+		v = value_new_cellrange_r (sheet, &r);
+		texpr = gnm_expr_top_new_constant (v);
+		dependent_managed_set_expr (&wbv->auto_expr.dep, texpr);
+		gnm_expr_top_unref (texpr);
+	}
+
+	wb_view_auto_expr_recalc (wbv);
 }
 
-static void
-wb_view_auto_expr_sheet (WorkbookView *wbv, gpointer *sheet)
-{
-	wbv->auto_expr_sheet = (Sheet *)sheet;
-
-	if (sheet != NULL)
-		wb_view_auto_expr_recalc (wbv);
-}
-
-
 static void
 wb_view_set_property (GObject *object, guint property_id,
 		      const GValue *value, GParamSpec *pspec)
@@ -761,17 +685,11 @@ wb_view_set_property (GObject *object, guint property_id,
 	case PROP_AUTO_EXPR_MAX_PRECISION:
 		wb_view_auto_expr_precision (wbv, g_value_get_boolean (value));
 		break;
-	case PROP_AUTO_EXPR_TEXT:
-		wb_view_auto_expr_text (wbv, g_value_get_string (value));
+	case PROP_AUTO_EXPR_VALUE:
+		wb_view_auto_expr_value (wbv, g_value_get_boxed (value));
 		break;
-	case PROP_AUTO_EXPR_ATTRS:
-		wb_view_auto_expr_attrs (wbv, g_value_peek_pointer (value));
-		break;
-	case PROP_AUTO_EXPR_CELL:
-		wb_view_auto_expr_cell (wbv, g_value_get_pointer (value));
-		break;
-	case PROP_AUTO_EXPR_SHEET:
-		wb_view_auto_expr_sheet (wbv, g_value_get_pointer (value));
+	case PROP_AUTO_EXPR_EVAL_POS:
+		wb_view_auto_expr_eval_pos (wbv, g_value_get_pointer (value));
 		break;
 	case PROP_SHOW_HORIZONTAL_SCROLLBAR:
 		wbv->show_horizontal_scrollbar = !!g_value_get_boolean (value);
@@ -818,25 +736,16 @@ wb_view_get_property (GObject *object, guint property_id,
 
 	switch (property_id) {
 	case PROP_AUTO_EXPR_FUNC:
-		g_value_set_pointer (value, wbv->auto_expr_func);
+		g_value_set_pointer (value, wbv->auto_expr.func);
 		break;
 	case PROP_AUTO_EXPR_DESCR:
-		g_value_set_string (value, wbv->auto_expr_descr);
+		g_value_set_string (value, wbv->auto_expr.descr);
 		break;
 	case PROP_AUTO_EXPR_MAX_PRECISION:
-		g_value_set_boolean (value, wbv->auto_expr_use_max_precision);
+		g_value_set_boolean (value, wbv->auto_expr.use_max_precision);
 		break;
-	case PROP_AUTO_EXPR_TEXT:
-		g_value_set_string (value, wbv->auto_expr_text);
-		break;
-	case PROP_AUTO_EXPR_ATTRS:
-		g_value_set_boxed (value, wbv->auto_expr_attrs);
-		break;
-	case PROP_AUTO_EXPR_CELL:
-		g_value_set_pointer (value, &wbv->auto_expr_cell);
-		break;
-	case PROP_AUTO_EXPR_SHEET:
-		g_value_set_pointer (value, wbv->auto_expr_sheet);
+	case PROP_AUTO_EXPR_VALUE:
+		g_value_set_boxed (value, wbv->auto_expr.value);
 		break;
 	case PROP_SHOW_HORIZONTAL_SCROLLBAR:
 		g_value_set_boolean (value, wbv->show_horizontal_scrollbar);
@@ -898,33 +807,14 @@ wb_view_dispose (GObject *object)
 			g_warning ("Unexpected left-over controls");
 	}
 
+	/* The order of these are important.  Make sure not to leak the value.  */
+	wb_view_auto_expr_descr (wbv, NULL);
+	wb_view_auto_expr_eval_pos (wbv, NULL);
+	wb_view_auto_expr_func (wbv, NULL);
+	wb_view_auto_expr_value (wbv, NULL);
+
 	wb_view_detach_from_workbook (wbv);
 
-	parent_class->dispose (object);
-}
-
-
-static void
-wb_view_finalize (GObject *object)
-{
-	WorkbookView *wbv = WORKBOOK_VIEW (object);
-
-	if (wbv->auto_expr_func) {
-		gnm_func_unref (wbv->auto_expr_func);
-		wbv->auto_expr_func = NULL;
-	}
-
-	g_free (wbv->auto_expr_descr);
-	wbv->auto_expr_descr = NULL;
-
-	g_free (wbv->auto_expr_text);
-	wbv->auto_expr_text = NULL;
-
-	if (wbv->auto_expr_attrs) {
-		pango_attr_list_unref (wbv->auto_expr_attrs);
-		wbv->auto_expr_attrs = NULL;
-	}
-
 	if (wbv->current_style != NULL) {
 		gnm_style_unref (wbv->current_style);
 		wbv->current_style = NULL;
@@ -935,9 +825,10 @@ wb_view_finalize (GObject *object)
 		wbv->in_cell_combo = NULL;
 	}
 
-	parent_class->finalize (object);
+	parent_class->dispose (object);
 }
 
+
 static void
 workbook_view_class_init (GObjectClass *gobject_class)
 {
@@ -945,7 +836,6 @@ workbook_view_class_init (GObjectClass *gobject_class)
 
 	gobject_class->set_property = wb_view_set_property;
 	gobject_class->get_property = wb_view_get_property;
-	gobject_class->finalize = wb_view_finalize;
 	gobject_class->dispose = wb_view_dispose;
 
 	/* FIXME?  Make a boxed type.  */
@@ -977,37 +867,21 @@ workbook_view_class_init (GObjectClass *gobject_class)
 				       G_PARAM_READWRITE));
         g_object_class_install_property
 		(gobject_class,
-		 PROP_AUTO_EXPR_TEXT,
-		 g_param_spec_string ("auto-expr-text",
-				      _("Auto-expression text"),
-				      _("Displayed text for the automatically computed sheet function."),
-				      NULL,
-				      GSF_PARAM_STATIC |
-				      G_PARAM_READWRITE));
+		 PROP_AUTO_EXPR_VALUE,
+		 g_param_spec_boxed ("auto-expr-value",
+				     _("Auto-expression value"),
+				     _("The current value of the auto-exprssion."),
+				     gnm_value_get_type (),
+				     GSF_PARAM_STATIC |
+				     G_PARAM_READWRITE));
         g_object_class_install_property
 		(gobject_class,
-		 PROP_AUTO_EXPR_ATTRS,
-		 g_param_spec_boxed ("auto-expr-attrs",
-				     _("Auto-expression Attributes"),
-				     _("Text attributes for the automatically computed sheet function."),
-				     PANGO_TYPE_ATTR_LIST,
-				     GSF_PARAM_STATIC | G_PARAM_READWRITE));
-        g_object_class_install_property
-		(gobject_class,
-		 PROP_AUTO_EXPR_CELL,
-		 g_param_spec_pointer ("auto-expr-cell",
-				       _("Auto-expression Cell Position"),
-				       _("The address of the cell to be shown."),
+		 PROP_AUTO_EXPR_EVAL_POS,
+		 g_param_spec_pointer ("auto-expr-eval-pos",
+				       _("Auto-expression position"),
+				       _("The cell position to track."),
 				       GSF_PARAM_STATIC |
-				       G_PARAM_READWRITE));
-        g_object_class_install_property
-		(gobject_class,
-		 PROP_AUTO_EXPR_SHEET,
-		 g_param_spec_pointer ("auto-expr-sheet",
-				       _("Auto-expression Sheet"),
-				       _("The sheet on which the cell resides."),
-				       GSF_PARAM_STATIC |
-				       G_PARAM_READWRITE));
+				       G_PARAM_WRITABLE));
         g_object_class_install_property
 		(gobject_class,
 		 PROP_SHOW_HORIZONTAL_SCROLLBAR,
@@ -1126,20 +1000,17 @@ workbook_view_new (Workbook *wb)
 	wbv->current_sheet_view = NULL;
 
 	/* Set the default operation to be performed over selections */
-	wbv->auto_expr_func = gnm_func_lookup ("sum", NULL);
-	if (wbv->auto_expr_func)
-		gnm_func_ref (wbv->auto_expr_func);
-	wbv->auto_expr_descr = g_strdup (_("Sum"));
-	wbv->auto_expr_text = NULL;
-	wbv->auto_expr_attrs = NULL;
-	wbv->auto_expr_use_max_precision = FALSE;
+	wbv->auto_expr.func = gnm_func_lookup ("sum", NULL);
+	if (wbv->auto_expr.func)
+		gnm_func_ref (wbv->auto_expr.func);
+	wbv->auto_expr.descr = g_strdup (_("Sum"));
+	wbv->auto_expr.value = NULL;
+	wbv->auto_expr.use_max_precision = FALSE;
+	dependent_managed_init (&wbv->auto_expr.dep, NULL);
 
 	for (i = 0 ; i < workbook_sheet_count (wb); i++)
 		wb_view_sheet_add (wbv, workbook_sheet_by_index (wb, i));
 
-	WORKBOOK_VIEW_FOREACH_CONTROL (wbv, wbc,
-		wb_view_init_control (wbc););
-
 	return wbv;
 }
 
